@@ -7,21 +7,49 @@ from .cache import Cache
 class _Link(object):
 
     __slots__ = (
-        'key', 'value', 'expire', 'size',
+        'key', 'expire', 'size',
         'ttl_prev', 'ttl_next',
         'lru_prev', 'lru_next'
     )
 
-    def unlink(self):
+    def __getstate__(self):
+        if hasattr(self, 'key'):
+            return (self.key, self.expire, self.size)
+        else:
+            return None
+
+    def __setstate__(self, state):
+        self.key, self.expire, self.size = state
+
+    def insert_lru(self, next):
+        self.lru_next = next
+        self.lru_prev = prev = next.lru_prev
+        prev.lru_next = next.lru_prev = self
+
+    def insert_ttl(self, next):
+        self.ttl_next = next
+        self.ttl_prev = prev = next.ttl_prev
+        prev.ttl_next = next.ttl_prev = self
+
+    def insert(self, next):
+        self.insert_lru(next)
+        self.insert_ttl(next)
+
+    def unlink_lru(self):
+        lru_next = self.lru_next
+        lru_prev = self.lru_prev
+        lru_prev.lru_next = lru_next
+        lru_next.lru_prev = lru_prev
+
+    def unlink_ttl(self):
         ttl_next = self.ttl_next
         ttl_prev = self.ttl_prev
         ttl_prev.ttl_next = ttl_next
         ttl_next.ttl_prev = ttl_prev
 
-        lru_next = self.lru_next
-        lru_prev = self.lru_prev
-        lru_prev.lru_next = lru_next
-        lru_next.lru_prev = lru_prev
+    def unlink(self):
+        self.unlink_lru()
+        self.unlink_ttl()
 
 
 class _NestedTimer(object):
@@ -61,9 +89,10 @@ class TTLCache(Cache):
     def __init__(self, maxsize, ttl, timer=time.time, missing=None,
                  getsizeof=None):
         Cache.__init__(self, maxsize, missing, getsizeof)
-        root = self.__root = _Link()
+        self.__root = root = _Link()
         root.ttl_prev = root.ttl_next = root
         root.lru_prev = root.lru_next = root
+        self.__links = {}
         self.__timer = _NestedTimer(timer)
         self.__ttl = ttl
 
@@ -71,7 +100,7 @@ class TTLCache(Cache):
         # prevent item reordering/expiration
         return '%s(%r, maxsize=%d, currsize=%d)' % (
             self.__class__.__name__,
-            [(key, cache_getitem(self, key).value) for key in self],
+            [(key, cache_getitem(self, key)) for key in self],
             self.maxsize,
             self.currsize,
         )
@@ -80,63 +109,44 @@ class TTLCache(Cache):
                     cache_getitem=Cache.__getitem__,
                     cache_missing=Cache.__missing__):
         with self.__timer as time:
-            link = cache_getitem(self, key)
+            value = cache_getitem(self, key)
+            link = self.__links[key]
             if link.expire < time:
-                return cache_missing(self, key).value
-            next = link.lru_next
-            prev = link.lru_prev
-            prev.lru_next = next
-            next.lru_prev = prev
-            link.lru_next = root = self.__root
-            link.lru_prev = tail = root.lru_prev
-            tail.lru_next = root.lru_prev = link
-            return link.value
+                return cache_missing(self, key)
+            link.unlink_lru()
+            link.insert_lru(self.__root)
+            return value
 
     def __setitem__(self, key, value,
-                    cache_contains=Cache.__contains__,
-                    cache_getitem=Cache.__getitem__,
                     cache_setitem=Cache.__setitem__,
                     cache_getsizeof=Cache.getsizeof):
         with self.__timer as time:
             self.expire(time)
-            if cache_contains(self, key):
-                oldlink = cache_getitem(self, key)
+            cache_setitem(self, key, value)
+            try:
+                link = self.__links[key]
+            except KeyError:
+                link = self.__links[key] = _Link()
             else:
-                oldlink = None
-            link = _Link()
+                link.unlink()
             link.key = key
-            link.value = value
             link.expire = time + self.__ttl
             link.size = cache_getsizeof(self, value)
-            cache_setitem(self, key, link)
-            if oldlink:
-                oldlink.unlink()
-            link.ttl_next = root = self.__root
-            link.ttl_prev = tail = root.ttl_prev
-            tail.ttl_next = root.ttl_prev = link
-            link.lru_next = root
-            link.lru_prev = tail = root.lru_prev
-            tail.lru_next = root.lru_prev = link
+            link.insert(self.__root)
 
-    def __delitem__(self, key,
-                    cache_contains=Cache.__contains__,
-                    cache_getitem=Cache.__getitem__,
-                    cache_delitem=Cache.__delitem__):
+    def __delitem__(self, key, cache_delitem=Cache.__delitem__):
         with self.__timer as time:
             self.expire(time)
-            if not cache_contains(self, key):
-                raise KeyError(key)
-            link = cache_getitem(self, key)
             cache_delitem(self, key)
-            link.unlink()
+            links = self.__links
+            links[key].unlink()
+            del links[key]
 
-    def __contains__(self, key,
-                     cache_contains=Cache.__contains__,
-                     cache_getitem=Cache.__getitem__):
+    def __contains__(self, key):
         with self.__timer as time:
-            if not cache_contains(self, key):
+            if key not in self.__links:
                 return False
-            elif cache_getitem(self, key).expire < time:
+            elif self.__links[key].expire < time:
                 return False
             else:
                 return True
@@ -161,6 +171,25 @@ class TTLCache(Cache):
                 head = head.ttl_next
         return cache_len(self) - expired
 
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        root = self.__root
+        links = state['__links'] = [(root, root)]
+        lru, ttl = root.lru_next, root.ttl_next
+        while lru is not root:
+            links.append((lru, ttl))
+            lru = lru.lru_next
+            ttl = ttl.ttl_next
+        return state
+
+    def __setstate__(self, state):
+        links = state.pop('__links')
+        count = len(links)
+        for index, (lru, ttl) in enumerate(links):
+            lru.lru_prev, ttl.ttl_prev = links[index - 1]
+            lru.lru_next, ttl.ttl_next = links[(index + 1) % count]
+        self.__dict__.update(state)
+
     @property
     def currsize(self):
         root = self.__root
@@ -188,20 +217,15 @@ class TTLCache(Cache):
             time = self.__timer()
         root = self.__root
         head = root.ttl_next
+        links = self.__links
         cache_delitem = Cache.__delitem__
         while head is not root and head.expire < time:
             cache_delitem(self, head.key)
+            del links[head.key]
             next = head.ttl_next
             head.unlink()
             head = next
 
-    def getsizeof(self, value):
-        """Return the size of a cache element's value."""
-        if isinstance(value, _Link):
-            return value.size
-        else:
-            return Cache.getsizeof(self, value)
-
     def popitem(self):
         """Remove and return the `(key, value)` pair least recently used that
         has not already expired.
@@ -212,11 +236,9 @@ class TTLCache(Cache):
             root = self.__root
             link = root.lru_next
             if link is root:
-                raise KeyError('cache is empty')
+                raise KeyError('%s is empty' % self.__class__.__name__)
             key = link.key
-            Cache.__delitem__(self, key)
-            link.unlink()
-            return (key, link.value)
+            return (key, self.pop(key))
 
     # mixin methods
 
